/*
 * ext/Cgraph/Cgraph.c
 * 
 * by Nakamura Noritsugu
 * from 1997.11.24
 */

#include <cgraph.h>
#include "ruby.h"

static VALUE mCgraph;

static VALUE
cgraph_aorigin(VALUE obj, VALUE x, VALUE y)
{
  int c;
  c = cg_aorigin((float)NUM2DBL(x), (float)NUM2DBL(y));
  return INT2FIX(c);
}

static VALUE
cgraph_axis_enable(VALUE obj, VALUE axisEnable, VALUE numberEnable)
{
  int c;
  c = cg_axis_enable(NUM2INT(axisEnable), NUM2INT(numberEnable));
  return INT2FIX(c);
}

static VALUE
cgraph_centerlabel(VALUE obj, VALUE textstring, VALUE x, VALUE y,
				   VALUE rotate, VALUE flag)
{
  switch (TYPE(textstring)) {
  case T_STRING:
	cg_centerlabel(STR2CSTR(textstring), (float)NUM2DBL(x), (float)NUM2DBL(y),
				   (float)NUM2DBL(rotate), NUM2INT(flag));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
cgraph_closepath()
{
  int c;
  c = cg_closepath();
  return INT2FIX(c);
}

static VALUE
cgraph_coord_select(VALUE obj, VALUE marks, VALUE text)
{
  int c;
  c = cg_coord_select(NUM2INT(marks), NUM2INT(text));
  return INT2FIX(c);
}

static VALUE
cgraph_dash(VALUE obj, VALUE type, VALUE dashperiod)
{
  int c;
  c = cg_dash(NUM2INT(type), (float)NUM2DBL(dashperiod));
  return INT2FIX(c);
}

static VALUE
cgraph_fill()
{
  int c;
  c = cg_fill();
  return INT2FIX(c);
}

static VALUE
cgraph_eofill()
{
  int c;
  c = cg_eofill();
  return INT2FIX(c);
}

static VALUE
cgraph_font(VALUE obj, VALUE fontsize, VALUE type)
{
  int c;
  c = cg_font((float)NUM2DBL(fontsize), NUM2INT(type));
  return INT2FIX(c);
}

static VALUE
cgraph_fontbyname(VALUE obj, VALUE fontsize, VALUE fontname)
{
  int c;
  switch (TYPE(fontname)) {
  case T_STRING:
	c = cg_fontbyname((float)NUM2DBL(fontsize), STR2CSTR(fontname));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2FIX(c);
}

static VALUE
cgraph_get_output_filename()
{
  return (VALUE) cg_get_output_filename();
}

static VALUE
cgraph_gray(VALUE obj, VALUE lightness)
{
  int c;
  c = cg_gray((float)NUM2DBL(lightness));
  return INT2FIX(c);
}

static VALUE
cgraph_grayrgbcolor(VALUE obj, VALUE gray, VALUE red, VALUE green, VALUE blue)
{
  cg_grayrgbcolor((float)NUM2DBL(gray), (float)NUM2DBL(red),
					  (float)NUM2DBL(green), (float)NUM2DBL(blue));
  return Qnil;
}

static VALUE
cgraph_grestore()
{
  int c;
  c = cg_grestore();
  return INT2FIX(c);
}

static VALUE
cgraph_gsave()
{
  int c;
  c = cg_gsave();
  return INT2FIX(c);
}

static VALUE
cgraph_init(VALUE obj, VALUE rot, VALUE expand, VALUE scale)
{
  int c;
  c = cg_init(NUM2INT(rot), NUM2INT(expand), (float)NUM2DBL(scale));
  return INT2FIX(c);
}

static VALUE
cgraph_launch_preview(VALUE obj, VALUE flag)
{
  cg_launch_preview(NUM2INT(flag));
  return Qnil;
}

static VALUE
cgraph_linax_style(VALUE obj, VALUE precision, VALUE numoff, VALUE numdist,
				   VALUE tnposition, VALUE ticlen)
{
  int c;
  c = cg_linax_style(NUM2INT(precision), NUM2INT(numoff), NUM2INT(numdist),
				 NUM2INT(tnposition), NUM2INT(ticlen));
  return INT2FIX(c);
}

static VALUE
cgraph_line(VALUE obj, VALUE x, VALUE y)
{
  int c;
  c = cg_line((float)NUM2DBL(x), (float)NUM2DBL(y));
  return INT2FIX(c);
}

static VALUE
cgraph_linewidth(VALUE obj, VALUE width)
{
  int c;
  c = cg_linewidth((float)NUM2DBL(width));
  return INT2FIX(c);
}

static VALUE
cgraph_logax_style(VALUE obj, VALUE numsel, VALUE ticsel, VALUE numdist,
				   VALUE tnposition, VALUE ticlen, VALUE ticlen10, VALUE numformat)
{
  int c;
  c = cg_logax_style(NUM2INT(numsel), NUM2INT(ticsel), NUM2INT(numdist), NUM2INT(tnposition),
					 NUM2INT(ticlen), NUM2INT(ticlen10), NUM2INT(numformat));
  return INT2FIX(c);
}

static VALUE
cgraph_marker(VALUE obj, VALUE x, VALUE y, VALUE type, VALUE size)
{
  int c;
  c = cg_marker((float)NUM2DBL(x), (float)NUM2DBL(y), NUM2INT(type), (float)NUM2DBL(size));
  return INT2FIX(c);
}

static VALUE
cgraph_markergray(VALUE obj, VALUE peri_gray, VALUE inside_gray)
{
  cg_markergray((float)NUM2DBL(peri_gray), (float)NUM2DBL(inside_gray));
  return Qnil;
}

static VALUE
cgraph_markercolor(VALUE obj, VALUE peri_r, VALUE peri_g, VALUE peri_b,
				  VALUE inside_r, VALUE inside_g, VALUE inside_b)
{
  cg_markercolor((float)NUM2DBL(peri_r), (float)NUM2DBL(peri_g), (float)NUM2DBL(peri_b),
				(float)NUM2DBL(inside_r), (float)NUM2DBL(inside_g), (float)NUM2DBL(inside_b));
  return Qnil;
}

static VALUE
cgraph_markergraycolor(VALUE obj, VALUE peri_gray, VALUE peri_r, VALUE peri_g, VALUE peri_b,
				  VALUE inside_gray, VALUE inside_r, VALUE inside_g, VALUE inside_b)
{
  cg_markergraycolor((float)NUM2DBL(peri_gray), (float)NUM2DBL(peri_r), (float)NUM2DBL(peri_g), (float)NUM2DBL(peri_b),
				(float)NUM2DBL(inside_gray), (float)NUM2DBL(inside_r), (float)NUM2DBL(inside_g), (float)NUM2DBL(inside_b));
  return Qnil;
}

static VALUE
cgraph_mesh()
{
  int c;
  c = cg_mesh();
  return INT2FIX(c);
}

static VALUE
cgraph_move(VALUE obj, VALUE x, VALUE y)
{
  int c;
  c = cg_move((float)NUM2DBL(x), (float)NUM2DBL(y));
  return INT2FIX(c);
}

static VALUE
cgraph_pointer(VALUE obj, VALUE x, VALUE y, VALUE type, VALUE size)
{
  int c;
  c = cg_pointer((float)NUM2DBL(x), (float)NUM2DBL(y), NUM2INT(type), (float)NUM2DBL(size));
  return INT2FIX(c);
}

static VALUE
cgraph_postscript(VALUE obj, VALUE string)
{
  int c;
  switch (TYPE(string)) {
  case T_STRING:
	c = cg_postscript(STR2CSTR(string));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2FIX(c);
}

/*static VALUE
cgraph_reset()
{
  int c;
  c = cg_reset();
  return INT2FIX(c);
}*/

static VALUE
cgraph_rgbcolor(VALUE obj, VALUE red, VALUE green, VALUE blue)
{
  cg_rgbcolor((float)NUM2DBL(red), (float)NUM2DBL(green), NUM2INT(blue));
}

static VALUE
cgraph_rmove(VALUE obj, VALUE x, VALUE y)
{
  int c;
  c = cg_rmove((float)NUM2DBL(x), (float)NUM2DBL(y));
  return INT2FIX(c);
}

static VALUE
cgraph_rline(VALUE obj, VALUE x, VALUE y)
{
  int c;
  c = cg_rline((float)NUM2DBL(x), (float)NUM2DBL(y));
  return INT2FIX(c);
}

static VALUE
cgraph_rorigin(VALUE obj, VALUE x, VALUE y)
{
  int c;
  c = cg_rorigin((float)NUM2DBL(x), (float)NUM2DBL(y));
  return INT2FIX(c);
}

static VALUE
cgraph_setboundingbox(VALUE obj, VALUE bbox)
{
  int c;
  switch (TYPE(bbox)) {
  case T_STRING:
	c = cg_setboundingbox(STR2CSTR(bbox));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2FIX(c);
}

static VALUE
cgraph_settitle(VALUE obj, VALUE title)
{
  int c;
  switch (TYPE(title)) {
  case T_STRING:
	c = cg_settitle(STR2CSTR(title));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2FIX(c);
}

static VALUE
cgraph_setcreator(VALUE obj, VALUE creator)
{
  int c;
  switch (TYPE(creator)) {
  case T_STRING:
	c = cg_setcreator(STR2CSTR(creator));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2FIX(c);
}

static VALUE
cgraph_setprolog(VALUE obj, VALUE prolog)
{
  int c;
  switch (TYPE(prolog)) {
  case T_STRING:
	c = cg_setprolog(STR2CSTR(prolog));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2FIX(c);
}

static VALUE
cgraph_set_top_comments(VALUE obj, VALUE comments)
{
  int c;
  switch (TYPE(comments)) {
  case T_STRING:
	c = cg_set_top_comments(STR2CSTR(comments));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2FIX(c);
}

static VALUE
cgraph_set_output_filename(VALUE obj, VALUE filename)
{
  switch (TYPE(filename)) {
  case T_STRING:
	cg_set_output_filename(STR2CSTR(filename));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
cgraph_showpage()
{
  int c;
  cg_showpage();
  return INT2FIX(c);
}

static VALUE
cgraph_stroke()
{
  int c;
  cg_stroke();
  return INT2FIX(c);
}

static VALUE
cgraph_text(VALUE obj, VALUE x, VALUE y,
			VALUE dir, VALUE textstring)
{
  int c;
  switch (TYPE(textstring)) {
  case T_STRING:
	c = cg_text((float)NUM2DBL(x), (float)NUM2DBL(y), (float)NUM2DBL(dir), STR2CSTR(textstring));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return INT2FIX(c);
}

static VALUE
cgraph_textalign(VALUE obj, VALUE x, VALUE y, VALUE rotate,
				 VALUE textstring, VALUE xalign, VALUE yalign, VALUE flag)
{
  switch (TYPE(textstring)) {
  case T_STRING:
	cg_textalign((float)NUM2DBL(x), (float)NUM2DBL(y), (float)NUM2DBL(rotate),
				 STR2CSTR(textstring), NUM2INT(xalign), NUM2INT(yalign), NUM2INT(flag));
	break;
/*  case T_FIXNUM:

	break;*/
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
cgraph_useflexcolor(VALUE obj, VALUE flag)
{
  cg_useflexcolor(NUM2INT(flag));
  return Qnil;
}

static VALUE
cgraph_use_stdout(VALUE obj, VALUE flag)
{
  cg_use_stdout(NUM2INT(flag));
  return Qnil;
}

static VALUE
cgraph_xaxis(VALUE obj, VALUE size, VALUE min, VALUE max,
			 VALUE offset, VALUE ticsep, VALUE numsep)
{
  int c;
  c = cg_xaxis((float)NUM2DBL(size), (float)NUM2DBL(min), (float)NUM2DBL(max),
			   (float)NUM2DBL(offset), (float)NUM2DBL(ticsep), NUM2INT(numsep));
  return INT2FIX(c);
}

static VALUE
cgraph_yaxis(VALUE obj, VALUE size, VALUE min, VALUE max,
			 VALUE offset, VALUE ticsep, VALUE numsep)
{
  int c;
  c = cg_yaxis((float)NUM2DBL(size), (float)NUM2DBL(min), (float)NUM2DBL(max),
			   (float)NUM2DBL(offset), (float)NUM2DBL(ticsep), NUM2INT(numsep));
  return INT2FIX(c);
}

static VALUE
cgraph_xlabel(VALUE obj, VALUE textstring)
{
  switch (TYPE(textstring)) {
  case T_STRING:
	cg_xlabel(STR2CSTR(textstring));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
cgraph_ylabel(VALUE obj, VALUE textstring)
{
  switch (TYPE(textstring)) {
  case T_STRING:
	cg_ylabel(STR2CSTR(textstring));
	break;
  default:
	rb_raise(rb_eRuntimeError, "not valid value");
	break;
  }
  return Qnil;
}

static VALUE
cgraph_xlog(VALUE obj, VALUE size, VALUE min,
			VALUE max, VALUE offset)
{
  int c;
  c = cg_xlog((float)NUM2DBL(size), (float)NUM2DBL(min),
			  (float)NUM2DBL(max), (float)NUM2DBL(offset));
  return INT2FIX(c);
}

static VALUE
cgraph_ylog(VALUE obj, VALUE size, VALUE min,
			VALUE max, VALUE offset)
{
  int c;
  c = cg_ylog((float)NUM2DBL(size), (float)NUM2DBL(min),
			  (float)NUM2DBL(max), (float)NUM2DBL(offset));
  return INT2FIX(c);
}


void
Init_Cgraph()
{
    mCgraph = rb_define_module("Cgraph");
    rb_define_module_function(mCgraph, "aorigin", cgraph_aorigin, 2);
    rb_define_module_function(mCgraph, "axis_enable", cgraph_axis_enable, 2);
    rb_define_module_function(mCgraph, "centerlabel", cgraph_centerlabel, 5);
    rb_define_module_function(mCgraph, "closepath", cgraph_closepath, 0);
    rb_define_module_function(mCgraph, "coord_select", cgraph_coord_select, 2);
    rb_define_module_function(mCgraph, "dash", cgraph_dash, 2);
    rb_define_module_function(mCgraph, "fill", cgraph_fill, 0);
    rb_define_module_function(mCgraph, "eofill", cgraph_eofill, 0);
    rb_define_module_function(mCgraph, "font", cgraph_font, 2);
    rb_define_module_function(mCgraph, "fontbyname", cgraph_fontbyname, 2);
    rb_define_module_function(mCgraph, "get_output_filename", cgraph_get_output_filename, 0);
    rb_define_module_function(mCgraph, "gray", cgraph_gray, 1);
    rb_define_module_function(mCgraph, "grayrgbcolor", cgraph_grayrgbcolor, 4);
    rb_define_module_function(mCgraph, "grestore", cgraph_grestore, 0);
    rb_define_module_function(mCgraph, "gsave", cgraph_gsave, 0);
    rb_define_module_function(mCgraph, "init", cgraph_init, 3);
    rb_define_module_function(mCgraph, "launch_preview", cgraph_launch_preview, 1);
    rb_define_module_function(mCgraph, "linax_style", cgraph_linax_style, 5);
    rb_define_module_function(mCgraph, "line", cgraph_line, 2);
    rb_define_module_function(mCgraph, "linewidth", cgraph_linewidth, 1);
    rb_define_module_function(mCgraph, "logax_style", cgraph_logax_style, 7);
    rb_define_module_function(mCgraph, "marker", cgraph_marker, 4);
    rb_define_module_function(mCgraph, "markergray", cgraph_markergray, 2);
    rb_define_module_function(mCgraph, "markercolor", cgraph_markercolor, 6);
    rb_define_module_function(mCgraph, "markergraycolor", cgraph_markergraycolor, 8);
    rb_define_module_function(mCgraph, "mesh", cgraph_mesh, 0);
    rb_define_module_function(mCgraph, "move", cgraph_move, 2);
    rb_define_module_function(mCgraph, "pointer", cgraph_pointer, 4);
    rb_define_module_function(mCgraph, "postscript", cgraph_postscript, 1);
/*    rb_define_module_function(mCgraph, "reset", cgraph_reset, 0);*/
    rb_define_module_function(mCgraph, "rgbcolor", cgraph_rgbcolor, 3);
    rb_define_module_function(mCgraph, "rmove", cgraph_rmove, 2);
    rb_define_module_function(mCgraph, "rline", cgraph_rline, 2);
    rb_define_module_function(mCgraph, "rorigin", cgraph_rorigin, 2);
    rb_define_module_function(mCgraph, "setboundingbox", cgraph_setboundingbox, 1);
    rb_define_module_function(mCgraph, "settitle", cgraph_settitle, 1);
    rb_define_module_function(mCgraph, "setcreator", cgraph_setcreator, 1);
    rb_define_module_function(mCgraph, "setprolog", cgraph_setprolog, 1);
    rb_define_module_function(mCgraph, "set_top_comments", cgraph_set_top_comments, 1);
    rb_define_module_function(mCgraph, "set_output_filename", cgraph_set_output_filename, 1);
    rb_define_module_function(mCgraph, "showpage", cgraph_showpage, 0);
    rb_define_module_function(mCgraph, "stroke", cgraph_stroke, 0);
    rb_define_module_function(mCgraph, "text", cgraph_text, 4);
    rb_define_module_function(mCgraph, "textalign", cgraph_textalign, 7);
    rb_define_module_function(mCgraph, "useflexcolor", cgraph_useflexcolor, 1);
    rb_define_module_function(mCgraph, "use_stdout", cgraph_use_stdout, 1);
    rb_define_module_function(mCgraph, "xaxis", cgraph_xaxis, 6);
    rb_define_module_function(mCgraph, "yaxis", cgraph_yaxis, 6);
    rb_define_module_function(mCgraph, "xlabel", cgraph_xlabel, 1);
    rb_define_module_function(mCgraph, "ylabel", cgraph_ylabel, 1);
    rb_define_module_function(mCgraph, "xlog", cgraph_xlog, 4);
    rb_define_module_function(mCgraph, "ylog", cgraph_ylog, 4);

	rb_define_const(mCgraph, "Landscape", INT2FIX(LANDSCAPE));
	rb_define_const(mCgraph, "Portrait",  INT2FIX(PORTRAIT));
}
