#!/usr/local/bin/ruby
#
# ./TFopt-lrLog.rb > TFopt-lrLog.eps
#

require "Cgraph"
include Cgraph

comment= "% TFopt-lrLog.c [94-10-13, Izumi Ohzawa]\n% Comparison of L-R optimal TF [Log axes] from XT spectrum peaks.\n"

vLmin = 0.5
vLmax = 10.0
vRmin = 0.5
vRmax = 10.0

markersize = 8.0

filename = "TFampspect.dat"

ratio = 1.5
slopeH = ratio
slopeL = 1.0 / ratio

rot = 1
expand = 1
scale = 1.0

use_stdout(1)

setcreator($0)
settitle("Log-log scatter plot from XY data file")
setboundingbox("0 0 270 270")
set_top_comments(comment)
init(rot, expand, scale)

aorigin(0.5, 0.5)
font(18.0, 30)
grayrgbcolor(0.0,  1.0, 1.0, 0.4)
logax_style(0xb, 0x7ff, 4,  3, 4, 8, 0)

xlog( 3.0, 0.5, 10.0, 0.0)
ylog( 3.0, 0.5, 10.0, 0.0)

mesh
font(22.0, 30)

grayrgbcolor(0.0,  0.7, 0.7, 0.7)
linewidth(0.6)
move(0.5, 0.5)
line(10.0, 10.0)
stroke

dash(6, 0.1364)
move(vLmin * ratio, vRmin)
line(vLmax, vRmax / ratio)
move(vLmin, vRmin * ratio)
line(vLmax/ratio, vRmax)
stroke
dash(0, 1.0)

ninside = 0
ntotal = 0
linewidth(1.0)

markergraycolor(0.0, 1.0, 1.0, 1.0,
				1.0, 1.0, 0.0, 0.0)
marktype = 0

f = open(filename, "r")
begin
  while line = f.gets
	if line =~ /^#/ || /^%/ || /^;/
	  next
	else
 	  if line =~ /(\S+)[\s\t]+(\S+)/
 		tfL = $1.to_f; tfR = $2.to_f
 		ntotal += 1
 		marker(tfL, tfR, marktype, markersize)
 		yH = slopeH * tfL
 		yL = slopeL * tfL
 	    if tfR <= yH && tfR >= yL
 		  ninside += 1
 		end
	  else
		STDERR.print "skip : #{line}\n"
 	  end
	end
  end
#rescue
#  STDERR.print "Can't open data file: #{filename}\n"
ensure
  f.close
end

STDERR.printf "%d of %d points are within a factor of %6.2f\n",
  ninside, ntotal, ratio

showpage
